## tpad version 1.3
# WinXP (TM) enhanced Notepad clone written in Tcl/Tk.
# BSD Text Formatter plugin
# Copyright (C) 2004  Antonio Bonifati
#
# Please see the main file tpad.tcl for licence and disclaimer
#
## last revised on: $Date: 2004/08/31 $

package provide tpad 1.3

# make sure the msgcat namespace exists
namespace eval ::msgcat {}

namespace eval ::tpad::BSDfmt {
  namespace export processText display

  namespace import ::msgcat::mc ::msgcat::mcmax
}

proc ::tpad::BSDfmt::DisplaySettings {pn} {
  variable ::tpad::$pn\::p
  foreach v {width excess ending} {
    variable $pn\::$v
  }

  set m $p.fmt

  $m.len.width.spin set $width
  $m.len.excess.spin set $excess

  set w $m.para.sentence.entry
  $w configure -state normal
  $w delete 0 end
  $w insert 0 $ending

  EnableCheckbuttons $pn
}

proc ::tpad::BSDfmt::WriteSettings {pn} {
  variable ::tpad::$pn\::p
  foreach v {width excess ending} {
    variable $pn\::$v
  }

  set m $p.fmt
  set w $m.len
 
  if {[$w.width.spin get]=={}} {
    tk_messageBox -title [mc "BSD Text formatter"] -icon error -parent $m \
      -message [mc "line width can't be empty"]
    ::tk::TabToWindow $w.width.spin
    return no
  }

  if {[$w.excess.spin get] =={}} {
    tk_messageBox -title [mc "BSD Text formatter"] -icon error -parent $m \
      -message [mc "excess chars can't be empty"]
    ::tk::TabToWindow $w.excess.spin
    return no
  }

  set width [$w.width.spin get]
  set excess [$w.excess.spin get]
  set ending [$m.para.sentence.entry get]
  return yes
}

proc ::tpad::BSDfmt::InitSettings {pn} {
  namespace eval $pn {}

  foreach varname [info vars ::tpad::BSDfmt::*] {
    set value [set $varname]
    set var [namespace tail $varname] ;# unqualified varname

    variable $pn\::$var $value
  }
}

proc ::tpad::BSDfmt::processText {pn} {
  variable ::tpad::$pn\::p
  foreach v \
      {width excess center nonroff mail spaces ending replace para range} {
    variable $pn\::$v
  }

  # generate the most simple command

  set cmd [list $::tpad::fmt]

  set opts {}
  if {$center} {append opts c}
  if {$mail} {append opts m}
  if {$nonroff} {append opts n}
  if {$para} {append opts p}
  if {$spaces} {append opts s}
  if {$opts!={}} {lappend cmd -$opts}

  if {$replace} {lappend cmd -l 8}

  if {$ending!={.?!}} {
    lappend cmd -d $ending
  }

  if {$width!=65 || $excess!=10} {
    lappend cmd $width
  }

  if {$excess!=10} {
    lappend cmd [expr $width+$excess]
  }

  # execute it and capture output

  set w $p.text
  switch $range {
    all {
      set fmtOutput [eval exec $cmd << {[$w get 1.0 end]}]

      $w configure -autosep no
      $w edit separator

      $w delete 1.0 end
      $w insert 1.0 $fmtOutput

      $w edit separator
      $w configure -autosep yes
    }

    sel {
      set keepnl [expr {[
        $w get {sel.last - 1 char}] == "\n" ? {-keepnewline} : {}
      }]

      set fmtOutput [eval exec $keepnl $cmd << {[selection get]}]
      set chars [string length $fmtOutput]

      $w configure -autosep no
      $w edit separator

      set first [$w index sel.first]
      $w delete $first sel.last
      $w insert insert $fmtOutput

      $w edit separator
      $w configure -autosep yes

      $w tag add sel $first [$w index "$first + $chars chars"]
    }
  }
}

proc ::tpad::BSDfmt::EnableCheckbuttons {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::center

  set m $p.fmt
  set state [expr {$center ? {disabled} : {normal}}]

  foreach wid {len.excess.label len.excess.spin line.n line.m line.s \
    line.l para.p para.sentence.label para.sentence.entry} {
    $m.$wid configure -state $state
  }
}

proc ::tpad::BSDfmt::AutosetRange {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::range

  set m $p.fmt
  if { [catch {$p.text index sel.first}] } {
    set range all
    $m.txt.sel configure -state disabled
  } else {
    set range sel
    $m.txt.sel configure -state normal
  }
}

proc ::tpad::BSDfmt::display {pn} {
  variable ::tpad::$pn\::p

  if {![namespace exists $pn]} {
    InitSettings $pn
  }

  set m $p.fmt
  toplevel $m
  wm group $m $pn
  wm title $m [mc "BSD Text formatter"]
  wm transient $m $pn
  wm resizable $m no no
  # catch the kill of the window manager
  wm protocol $m WM_DELETE_WINDOW [namespace code "
    if {\[WriteSettings $pn\]} {
      set ::tpad::$pn\::format no
      destroy $m
    }
  "]

  labelframe $m.len -text [mc "Length Settings"] -padx 1.5m -pady 1.5m

  set w $m.len.width
  frame $w
  bind [::tpad::AmpWidget label $w.label -text [mc "line &width"] \
    -width [mcmax "line &width" "e&xcess chars"] -anchor w] \
    <<AltUnderlined>> [list ::tk::TabToWindow $w.spin]
  spinbox $w.spin -from 1 -to 2000 -width 6 -exportsel no \
    -validate key -vcmd [namespace code {string is integer %P}]
  pack $w.label $w.spin -side left

  set w $m.len.excess
  frame $w
  bind [::tpad::AmpWidget label $w.label -text [mc "e&xcess chars"] \
    -width [mcmax "line &width" "e&xcess chars"] -anchor w] \
    <<AltUnderlined>> [list ::tk::TabToWindow $w.spin]
  spinbox $w.spin -from 0 -to 1000 -width 6 -exportsel no \
    -validate key -vcmd [namespace code {string is integer %P}]
  pack $w.label $w.spin -side left

  pack $m.len.width $m.len.excess -anchor w

  set w $m.line
  labelframe $w -text [mc "Line Format Settings"] -padx 1.5m -pady 1.5m
  ::tpad::AmpWidget checkbutton $w.c -var ::tpad::BSDfmt::$pn\::center \
    -text [mc "&center the text line by line"] -onv yes -offv no \
    -comm [namespace code "EnableCheckbuttons $pn"]
  ::tpad::AmpWidget checkbutton $w.n -var ::tpad::BSDfmt::$pn\::nonroff \
    -text [mc "format lines begi&nning with `.'"] -onv yes -offv no
  ::tpad::AmpWidget checkbutton $w.m -text [mc "format &mail headers lines"] \
    -var ::tpad::BSDfmt::$pn\::mail -onv yes -offv no
  ::tpad::AmpWidget checkbutton $w.s -var ::tpad::BSDfmt::$pn\::spaces \
    -text [mc "collapse white&space inside lines"] -onv yes -offv no
  ::tpad::AmpWidget checkbutton $w.l -var ::tpad::BSDfmt::$pn\::replace \
    -text [mc "rep&lace spaces with tabs at the start"] \
    -onv yes -offv no
  pack $w.c $w.n $w.m $w.s $w.l -anchor w

  set w $m.para
  labelframe $w -text [mc "Paragraph Format Settings"] -padx 1.5m -pady 1.5m
  ::tpad::AmpWidget checkbutton $w.p -var ::tpad::BSDfmt::$pn\::para \
    -text [mc "allow indented &paragraphs"] -onv yes -offv no

  set w $w.sentence
  frame $w
  bind [::tpad::AmpWidget label $w.label -text [mc "sentence en&ding chars"] \
    -width [mcmax "sentence en&ding chars"] -anchor w] \
    <<AltUnderlined>> [list ::tk::TabToWindow $w.entry]
  entry $w.entry -width 10 -exportsel no
  pack $w.label $w.entry -side left

  pack $w $m.para.p -anchor w

  set w $m.txt
  labelframe $w -text [mc "Format Range"] -padx 1.5m -pady 1.5m

  ::tpad::AmpWidget radiobutton $w.all -var ::tpad::BSDfmt::$pn\::range \
    -value all -text [mc "Ent&ire text"]
  ::tpad::AmpWidget radiobutton $w.sel -var ::tpad::BSDfmt::$pn\::range \
    -value sel -text [mc S&election]
  AutosetRange $pn
  pack $w.all $w.sel -side left

  set w $m.but
  frame $w
  ::tpad::AmpWidget button $w.format -text [mc &Format] -comm \
  [namespace code "
    if {\[WriteSettings $pn\]} {
      processText $pn
    }
  "]
  ::tpad::AmpWidget button $w.restore -text [mc "&Restore options"] -comm [
    namespace code "InitSettings $pn; DisplaySettings $pn"
  ]
  ::tpad::AmpWidget button $w.close -text [mc Cl&ose] -comm [namespace code "
    if {\[WriteSettings $pn\]} {
      set ::tpad::$pn\::format no
      focus $p.text
      after idle {destroy $m}
    }
  "]
  pack $w.format $w.restore $w.close -side left -exp yes

  DisplaySettings $pn
  pack $m.len $m.line $m.para $m.txt $m.but -padx 10 -pady 6 -fill x

  bind $m <Alt-Key> [list ::tpad::AltKeyInDialog $m %A]
  bind $m <Key-Escape> [list ::tk::ButtonInvoke $m.but.close]

  ::tpad::PlaceWindow $pn $m
}
