## tpad version 1.3
# WinXP (TM) enhanced Notepad clone written in Tcl/Tk.
# GNU Text Formatter plugin
# Copyright (C) 2004  Antonio Bonifati
#
# Please see the main file tpad.tcl for licence and disclaimer
#
## last revised on: $Date: 2004/08/31 $

package provide tpad 1.3

# make sure the msgcat namespace exists
namespace eval ::msgcat {}

namespace eval ::tpad::GNUfmt {
  namespace export processText display

  namespace import ::msgcat::mc ::msgcat::mcmax
}

proc ::tpad::GNUfmt::DisplaySettings {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::width
  variable $pn\::prefix

  set m $p.fmt
  $m.len.spin set $width

  set w $m.line.prefix.entry
  $w delete 0 end
  $w insert 0 $prefix
}

proc ::tpad::GNUfmt::WriteSettings {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::width
  variable $pn\::prefix

  set m $p.fmt
  set w $m.len.spin

  if {[$w get]=={}} {
    tk_messageBox -title [mc "GNU Text formatter"] -icon error -parent $m \
       -message [mc "maximum line width can't be empty"]
    ::tk::TabToWindow $w
    return no
  }

  set width [$w get]
  set prefix [$m.line.prefix.entry get]
  return yes
}

proc ::tpad::GNUfmt::InitSettings {pn} {
  namespace eval $pn {}

  foreach varname [info vars ::tpad::GNUfmt::*] {
    set value [set $varname]
    set var [namespace tail $varname] ;# unqualified varname

    variable $pn\::$var $value
  }
}

proc ::tpad::GNUfmt::processText {pn} {
  variable ::tpad::$pn\::p
  foreach v {width uniform prefix crown tagged split range} {
    variable $pn\::$v
  }

  # generate the most simple command

  set cmd [list $::tpad::fmt]

  if {$width!=75} {
    lappend cmd -$width
  }

  set opts {}
  if {$crown} {append opts c}
  if {$split} {append opts s}
  if {$tagged} {append opts t}
  if {$uniform} {append opts u}
  if {$opts!={}} {lappend cmd -$opts}

  if {$prefix!={}} {
    lappend cmd -p $prefix
  }

  # execute it and capture output

  set w $p.text
  switch $range {
    all {
      set fmtOutput [eval exec $cmd << {[$w get 1.0 end]}]

      $w configure -autosep no
      $w edit separator

      $w delete 1.0 end
      $w insert 1.0 $fmtOutput

      $w edit separator
      $w configure -autosep yes
    }

    sel {
      set keepnl [expr {
        [$w get {sel.last - 1 char}] == "\n" ? {-keepnewline} : {}
      }]

      set fmtOutput [eval exec $keepnl $cmd << {[selection get]}]
      set chars [string length $fmtOutput]

      $w configure -autosep no
      $w edit separator

      set first [$w index sel.first]
      $w delete $first sel.last
      $w insert insert $fmtOutput

      $w edit separator
      $w configure -autosep yes

      $w tag add sel $first [$w index "$first + $chars chars"]
    }
  }
}

proc ::tpad::GNUfmt::AutosetRange {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::range

  set m $p.fmt
  if { [catch {$p.text index sel.first}] } {
    set range all
    $m.txt.sel configure -state disabled
  } else {
    set range sel
    $m.txt.sel configure -state normal
  }
}

proc ::tpad::GNUfmt::display {pn} {
  variable ::tpad::$pn\::p

  if {![namespace exists $pn]} {
    InitSettings $pn
  }

  set m $p.fmt
  toplevel $m
  wm group $m $pn
  wm title $m [mc "GNU Text formatter"]
  wm transient $m $pn
  wm resizable $m no no
  # catch the kill of the window manager
  wm protocol $m WM_DELETE_WINDOW [namespace code "
    if {\[WriteSettings $pn\]} {
      set ::tpad::$pn\::format no
      destroy $m
    }
  "]

  set w $m.len
  labelframe $w -text [mc "Length Settings"] -padx 1.5m -pady 1.5m
  bind [::tpad::AmpWidget label $w.label -text [mc "maximum line &width"] \
    -anchor w] <<AltUnderlined>> [list ::tk::TabToWindow $w.spin]
  spinbox $w.spin -from 1 -to 2000 -width 6 -exportsel no \
    -validate key -vcmd [namespace code {string is integer %P}]
  pack $w.label -side left
  pack $w.spin -side left -padx {1.5m 0}

  set w $m.line
  labelframe $w -text [mc "Line Format Settings"] -padx 1.5m -pady 1.5m
  ::tpad::AmpWidget checkbutton $w.u -text [mc "&uniform spacing"] \
    -onv yes -offv no -var ::tpad::GNUfmt::$pn\::uniform

  set w $w.prefix
  frame $w
  bind [::tpad::AmpWidget label $w.label \
    -text [mc "combine only if &prefix is"] -anchor w] \
    <<AltUnderlined>> [list ::tk::TabToWindow $w.entry]
  entry $w.entry -width 11 -exportsel no
  pack $w.label -side left
  pack $w.entry -side left -padx {1.5m 0}

  pack $m.line.u $w -anchor w

  set w $m.para
  labelframe $w -text [mc "Paragraph Format Settings"] -padx 1.5m -pady 1.5m
  ::tpad::AmpWidget checkbutton $w.c -text [mc "&crown margin"] \
    -onv yes -offv no -var ::tpad::GNUfmt::$pn\::crown
  ::tpad::AmpWidget checkbutton $w.t -text [mc "&tagged paragraph"] \
    -onv yes -offv no -var ::tpad::GNUfmt::$pn\::tagged
  ::tpad::AmpWidget checkbutton $w.s -text [mc "&split only, do not refill"] \
    -onv yes -offv no -var ::tpad::GNUfmt::$pn\::split 
  pack $w.c $w.t $w.s -anchor w

  set w $m.txt
  labelframe $w -text [mc "Format Range"] -padx 1.5m -pady 1.5m
  ::tpad::AmpWidget radiobutton $w.all -text [mc "ent&ire text"] \
    -value all -var ::tpad::GNUfmt::$pn\::range
  ::tpad::AmpWidget radiobutton $w.sel -text [mc s&election] \
    -value sel -var ::tpad::GNUfmt::$pn\::range
  AutosetRange $pn
  pack $w.all $w.sel -side left

  set w $m.but
  frame $w
  ::tpad::AmpWidget button $w.format -text [mc &Format] -comm \
  [namespace code "
    if {\[WriteSettings $pn\]} {
      processText $pn
    }
  "]
  ::tpad::AmpWidget button $w.restore -text [mc "&Restore options"] -comm [
    namespace code "InitSettings $pn; DisplaySettings $pn"
  ]
  ::tpad::AmpWidget button $w.close -text [mc Cl&ose] -comm [namespace code "
    if {\[WriteSettings $pn\]} {
      set ::tpad::$pn\::format no
      focus $p.text
      after idle {destroy $m}
    }
  "]
  pack $w.format $w.restore $w.close -side left -exp yes

  DisplaySettings $pn
  pack $m.len $m.line $m.para $m.txt $m.but -padx 10 -pady 6 -fill x

  bind $m <Alt-Key> [list ::tpad::AltKeyInDialog $m %A]
  bind $m <Key-Escape> [list ::tk::ButtonInvoke $m.but.close]

  ::tpad::PlaceWindow $pn $m
}
